from app.config import THEME_DEFAULT, LANG_DEFAULT, SECRET_KEY,PATH_LOCALES,PATH_UPLOADS
from lila.core.session import Session
from lila.core.request import Request
from lila.core.responses import JSONResponse
from lila.core.logger import Logger
import json
from pathlib import Path
import jwt
import hashlib
import secrets
from datetime import datetime, timedelta, date
from typing import Union, List, Set
from os import getenv
from PIL import Image

def theme(request: Request = None) -> str:
    if request:
        t = Session.getSessionValue(key="theme", request=request)
        if t:
            return Session.unsign(key="theme", request=request)
    t = THEME_DEFAULT
    return t

def lang(request: Request) -> str:
    l = LANG_DEFAULT or "en"
    session_lang = Session.getSessionValue(key="lang", request=request) 
    if session_lang:
        l = Session.unsign(key="lang", request=request) 
    return l


def translate(file_name: str, request: Request, lang_default: str = None) -> dict:
    current_lang = lang(request) if not lang_default else lang_default
    translations = {}
    file_path = Path(PATH_LOCALES) / f"{file_name}.json"

    try:
        with open(file=file_path, mode="r", encoding="utf-8") as f:
            data = json.load(f)
        for key, val in data.items():
            if current_lang in val:
                translations[key] = val[current_lang]
            else:
                translations[key] = key
        return translations
    except FileNotFoundError:
        print(f"{file_name}.json not found")
    except json.JSONDecodeError as e:
        print(f"{file_name} - {e}")

    return translations


def translate_(
    key: str,
    request: Request,
    file_name: str = "translations",
    lang_default: str = None,
) -> str:
    t = translate(file_name=file_name, request=request, lang_default=lang_default)
    if key in t:
        msg = t[key]
        return msg if msg not in [None, ""] else key
    return key


def generate_token_value(hex: int = 16) -> str:
    return hashlib.sha256(secrets.token_hex(hex).encode()).hexdigest()


def generate_token(name: str, value: str = None, minutes: int = 1440) -> str:
    options = {name: value, "exp": datetime.utcnow() + timedelta(minutes=minutes)}
    options[name] = value if value else generate_token_value()
    token = jwt.encode(options, SECRET_KEY, algorithm="HS256")
    return token


def get_token(token: str):
    try:
        token = token.strip().split(" ")
        if len(token) > 1:
            token = token[1]
            return jwt.decode(token, SECRET_KEY, algorithms=["HS256"])
        return JSONResponse(
            {"session": False, "message": "Invalid token"}, status_code=401
        )
    except jwt.ExpiredSignatureError:
        return JSONResponse(
            {"session": False, "message": "Token has expired"}, status_code=401
        )
    except jwt.InvalidTokenError:
        return JSONResponse(
            {"session": False, "message": "Invalid token"}, status_code=401
        )


def get_user_by_token(request: Request,key:str="user_id"):
    token = request.headers.get("Authorization")
    if not token:
        return JSONResponse(
            {"session": False, "message": "Invalid token"}, status_code=401
        )
    token = get_token(token=token)
    if isinstance(token, JSONResponse):
        return token
    user_id = token.get(key)
    if not user_id:
        return JSONResponse(
            {"session": False, "message": "Invalid token"}, status_code=401
        )
    return user_id


def convert_date_to_str(value):
    if isinstance(value, (date, datetime)):
        return value.isoformat() if value else None
    return value


ALLOWED_EXTENSIONS_ = {"txt", "pdf", "png", "jpg", "jpeg"}
MAX_FILE_SIZE_ = 10 * 1024 * 1024  # 10MB


async def upload(
    request: Request,
    name_file: Union[str, List[str]] = "file",
    UPLOAD_DIR: str = PATH_UPLOADS,
    ALLOWED_EXTENSIONS: Set[str] = ALLOWED_EXTENSIONS_,
    MAX_FILE_SIZE: int = MAX_FILE_SIZE_,
):
    """
    # English: Handles file upload with validations and translations
    # Español: Maneja la subida de archivos con validaciones y traducciones
    """
    try:
        # English: Validate HTTP method
        # Español: Validar método HTTP
        if request.method != "POST":
            return JSONResponse(
                {
                    "error": True,
                    "success": False,
                    "message": translate_("invalid_method", request),
                },
                status_code=405,
            )

        # English: Validate content type
        # Español: Validar tipo de contenido
        content_type = request.headers.get("content-type", "")
        if "multipart/form-data" not in content_type:
            return JSONResponse(
                {
                    "error": True,
                    "success": False,
                    "message": translate_("invalid_content_type", request),
                },
                status_code=400,
            )

        # English: Create upload directory if it doesn't exist
        # Español: Crear directorio de uploads si no existe
        upload_path = Path(UPLOAD_DIR)
        if not upload_path.exists():
            upload_path.mkdir(parents=True, exist_ok=True)

        try:
            form = await request.form()
            file = form.get(name_file)

            # English: Check if file exists in form
            # Español: Verificar si el archivo existe en el formulario
            if not file:
                return JSONResponse(
                    {
                        "error": True,
                        "success": False,
                        "message": translate_("file_not_found", request),
                    },
                    status_code=400,
                )

            # English: Handle multiple files
            # Español: Manejar múltiples archivos
            if isinstance(file, list):
                files = []
                for f in file:
                    # English: Validate file extension
                    # Español: Validar extensión del archivo
                    extension = f.filename.split(".")[-1].lower()
                    if extension not in ALLOWED_EXTENSIONS:
                        return JSONResponse(
                            {
                                "error": True,
                                "success": False,
                                "message": translate_("invalid_extension", request),
                            },
                            status_code=400,
                        )

                    # English: Validate file size
                    # Español: Validar tamaño del archivo
                    if f.size > MAX_FILE_SIZE:
                        return JSONResponse(
                            {
                                "error": True,
                                "success": False,
                                "message": translate_("file_too_large", request),
                            },
                            status_code=400,
                        )

                    # English: Validate file is not empty
                    # Español: Validar que el archivo no esté vacío
                    content = await f.read()
                    if len(content) == 0:
                        return JSONResponse(
                            {
                                "error": True,
                                "success": False,
                                "message": translate_("empty_file", request),
                            },
                            status_code=400,
                        )

                    # English: Save file with safe filename
                    # Español: Guardar archivo con nombre seguro
                    safe_filename = "".join(
                        c for c in f.filename if c.isalnum() or c in (" ", ".", "_")
                    ).rstrip()
                    file_path = upload_path / safe_filename

                    with open(file_path, "wb") as fp:
                        fp.write(content)

                    optimized_path = optimize_image(file_path,extension)
                    files.append(optimized_path)

                return JSONResponse(
                    {
                        "files": files,
                        "success": True,
                        "message": translate_("upload_success", request),
                    },
                    status_code=200,
                )

            # English: Handle single file (same validations as above)
            # Español: Manejar archivo único (mismas validaciones que arriba)
            extension = file.filename.split(".")[-1].lower()
            if extension not in ALLOWED_EXTENSIONS:
                return JSONResponse(
                    {
                        "error": True,
                        "success": False,
                        "message": translate_("invalid_extension", request),
                    },
                    status_code=400,
                )

            if file.size > MAX_FILE_SIZE:
                return JSONResponse(
                    {
                        "error": True,
                        "success": False,
                        "message": translate_("file_too_large", request),
                    },
                    status_code=400,
                )

            content = await file.read()
            if len(content) == 0:
                return JSONResponse(
                    {
                        "error": True,
                        "success": False,
                        "message": translate_("empty_file", request),
                    },
                    status_code=400,
                )

            safe_filename = "".join(
                c for c in file.filename if c.isalnum() or c in (" ", ".", "_")
            ).rstrip()
            file_path = upload_path / safe_filename

            with open(file_path, "wb") as fp:
                fp.write(content)

            file_path = optimize_image(file_path,extension)

            return JSONResponse(
                {
                    "file": str(file_path),
                    "success": True,
                    "message": translate_("upload_success", request),
                },
                status_code=200,
            )

        except Exception as e:
            Logger.error(str(e))
            return JSONResponse(
                {
                    "error": True,
                    "success": False,
                    "message": translate_("server_error", request),
                },
                status_code=400,
            )

    except Exception as e:
        Logger.error(str(e))
        return JSONResponse(
            {
                "error": True,
                "success": False,
                "message": translate_("server_error", request),
            },
            status_code=500,
        )


def optimize_image(file_path: Path, extension: str, max_width=1920, quality=75) -> Path:
    img = Image.open(file_path)
    if img.mode in ("RGBA", "P"):
        img = img.convert("RGB")

    if img.width > max_width:
        ratio = max_width / float(img.width)
        new_height = int(img.height * ratio)
        img = img.resize((max_width, new_height), Image.Resampling.LANCZOS)

    optimized_filename = f"{file_path.stem}.webp"
    optimized_path = file_path.parent / optimized_filename

    if extension.lower() in ["jpg", "jpeg", "png"]:
        img.save(optimized_path, format="WEBP", optimize=True, quality=quality)
    else:
        img.save(optimized_path, format=img.format, optimize=True, quality=quality)

    return optimized_path


def getenvironment(
    key: str, default: Union[str, int, bool] = None
) -> Union[str, int, bool]:
    return getenv(key, default) if default is not None else getenv(key)



from jsmin import jsmin
from cssmin import cssmin
from htmlmin import minify

import os

def main():
    success = "Files minified successfully"
    html_folder = "templates/html"
    public_folder = "static"
    js_folder = f"{public_folder}/js"
    css_folder = f"{public_folder}/css"
    
    for root, _, files in os.walk(js_folder):
        for file in files:
            if file.endswith('.js'):
                file_path = os.path.join(root, file)
                js(file_path)
    
    for root, _, files in os.walk(css_folder):
        for file in files:
            if file.endswith('.css'):
                file_path = os.path.join(root, file)
                css(file_path)
    
    for root, _, files in os.walk(html_folder):
        for file in files:
            if file.endswith('.html'):
                file_path = os.path.join(root, file)
                html(file_path)
    
    
    return success

def js(file_path: str) -> bool:
    try:
        with open(file_path, 'r', encoding='utf-8') as js_file:
            original_content = js_file.read()
        
        minified = jsmin(original_content)
        
        with open(file_path, 'w', encoding='utf-8') as js_file:
            js_file.write(minified)
        
        return True
    except Exception as e:
        print(f"Error minifying JS file {file_path}: {str(e)}")
        return False

def css(file_path: str) -> bool:
    try:
        with open(file_path, 'r', encoding='utf-8') as css_file:
            original_content = css_file.read()
        
        minified = cssmin(original_content)
        
        with open(file_path, 'w', encoding='utf-8') as css_file:
            css_file.write(minified)
        
        return True
    except Exception as e:
        print(f"Error minifying CSS file {file_path}: {str(e)}")
        return False

def html(file_path: str) -> bool:
    try:
        with open(file_path, 'r', encoding='utf-8') as html_file:
            original_content = html_file.read()
        
        minified = minify(original_content, remove_comments=True, remove_empty_space=True)
        
        with open(file_path, 'w', encoding='utf-8') as html_file:
            html_file.write(minified)
        
        return True
    except Exception as e:
        print(f"Error minifying HTML file {file_path}: {str(e)}")
        return False

if __name__ == "__main__":
    result = main()
    print(result)



import sys
import subprocess
from pathlib import Path
import time
import typer

REQUIRED_PACKAGES = [
    "selenium",
    "webdriver-manager",
    "beautifulsoup4"
]

def ensure_packages():
    for pkg in REQUIRED_PACKAGES:
        try:
            __import__(pkg.replace("-", "_"))
        except ImportError:
            print(f"📦Installing / Instalando {pkg}...")
            subprocess.check_call([sys.executable, "-m", "pip", "install", pkg])


ensure_packages()

from selenium import webdriver
from selenium.webdriver.chrome.service import Service
from webdriver_manager.chrome import ChromeDriverManager
from bs4 import BeautifulSoup

app = typer.Typer()

CACHE_DIR = Path("templates/html/react/cache/")
CACHE_DIR.mkdir(parents=True, exist_ok=True)

@app.command()
def cache_page_with_selenium(route: str = "/", url: str = 'http://localhost:5173/'):
    chrome_options = webdriver.ChromeOptions()
    chrome_options.add_argument("--headless")
    chrome_options.add_argument("--disable-gpu")
    chrome_options.add_argument("--no-sandbox")

    try:
        driver = webdriver.Chrome(
            service=Service(ChromeDriverManager().install()), 
            options=chrome_options
        )
    except Exception as e:
        print(f"❌ Error Selenium: {e}")
        return None

    full_url = f"{url}{route}"
    print(f"🌐 {full_url} with Selenium...")

    try:
        driver.get(full_url)
        time.sleep(5)

        body_html = driver.find_element("tag name", "body").get_attribute('innerHTML')

        soup = BeautifulSoup(body_html, "html.parser")

        root_div = soup.find("div", id="root")
        if root_div:
            root_div.unwrap()

        for script_tag in soup.find_all("script", src="/src/main.jsx"):
            script_tag.decompose()

        filename = CACHE_DIR / f"{route.strip('/') or 'index'}.html"
        filename.write_text(str(soup), encoding="utf-8")

        print(f"✅ Cache saved in {filename}")
        return filename

    except Exception as e:
        print(f"❌ Error : {e}")
        return None

    finally:
        driver.quit()

if __name__ == "__main__":
    app()



from lila.core.database import Database
from sqlalchemy.orm import Session

#English : Example connection to a sqlite database
#Español: Ejemplo de conexión a la base de datos con sqlite
config = {"type":"sqlite","database":"lila"} #lila.db
connection = Database(config=config)
connection.connect()



#English : Example connection to a mysql database
#Español: Ejemplo de conexión a la base de datos con mysql
#config = {"type":"mysql","host":"127.0.0.1","user":"root","password":"password","database":"db_test","auto_commit":True}
#connection = Database(config=config)
#connection.connect()
#mysql_connection = connection

#Use for orm sqlalchemy abstraction
db_session=connection.get_session()


from starlette.applications import Starlette
from lila.core.responses import HTMLResponse
from lila.core.templates import render
from starlette.middleware.cors import CORSMiddleware
from lila.core.logger import Logger 
import os 
from app.config import PATH_TEMPLATE_NOT_FOUND
from typing import List, Optional, Dict, Any


class App(Starlette):
    def __init__(
        self, 
        debug: bool = False, 
        routes: List = None, 
        cors: Optional[Dict[str, Any]] = None,
        middleware: List = None
    ):
        routes = routes or []
        
        middleware = middleware or []
        
        super().__init__(debug=debug, routes=routes, middleware=middleware)
        
        try:
            self.add_exception_handler(404, self._404_page)
            
            if cors:
                self.add_middleware(
                    CORSMiddleware,
                    allow_origins=cors.get("origin", ["*"]),
                    allow_credentials=cors.get("allow_credentials", True),
                    allow_methods=cors.get("allow_methods", ["*"]),
                    allow_headers=cors.get("allow_headers", ["*"]),
                )

            Logger.info("Application started successfully")
        except Exception as e:
            Logger.error(f"Error initializing application: {e}", exception=e)

    async def _404_page(self, request, exc):
        static_extensions = {'.js', '.css', '.jpg', '.jpeg', '.png', '.gif', '.ico', '.svg', '.woff', '.woff2', '.ttf', '.eot'}
        excluded_paths = {'public', 'static', 'assets', 'favicon.ico'}
        
        should_log = True
        path = request.url.path.lower()
        
        if any(path.endswith(ext) for ext in static_extensions) or any(excluded in path for excluded in excluded_paths):
            should_log = False
        
        if should_log:
            Logger.warning(f"404 Not Found: {request.url.path} - {await Logger.request(request=request)}")
        
        if os.path.exists(PATH_TEMPLATE_NOT_FOUND):
            response = render(request=request, template="404")
            return response
        else:
            html_content = "<h1>404 Not Found</h1><p>The requested resource was not found on this server.</p>"
            return HTMLResponse(content=html_content, status_code=404)


# lila/cli/scaffold.py
import shutil
import os
from importlib import resources
from pathlib import Path

SCAFFOLD_ITEMS = {
    "main.py": "main.py",
    "app": "app",
    "cli": "cli",
    "static": "static",
    "templates": "templates",
    "uploads": "uploads",
     ".env": ".env",
}

def copy_item(
    source_package_name, item_name_in_package, destination_base_path, item_name_in_dest
):
    try:
        with resources.path(source_package_name, item_name_in_package) as source_path:
            destination_path = Path(destination_base_path) / item_name_in_dest

            if not source_path.exists():
                print(
                    f"  ⚠️ Advertencia: El origen '{source_path}' no existe. Omitiendo. / "
                    f"Warning: Source '{source_path}' does not exist. Skipping."
                )
                return

            if source_path.is_dir():
                shutil.copytree(source_path, destination_path)
                print(
                    f"  📂 Directorio '{item_name_in_package}' copiado a '{destination_path}'. / "
                    f"Directory '{item_name_in_package}' copied to '{destination_path}'."
                )
            else:
                destination_path.parent.mkdir(parents=True, exist_ok=True)
                shutil.copy2(source_path, destination_path)
                print(
                    f"  📄 Archivo '{item_name_in_package}' copiado a '{destination_path}'. / "
                    f"File '{item_name_in_package}' copied to '{destination_path}'."
                )

    except FileNotFoundError:
        print(
            f"  ⚠️ Advertencia: Recurso '{source_package_name}/{item_name_in_package}' no encontrado. Omitiendo. / "
            f"Warning: Resource '{source_package_name}/{item_name_in_package}' not found. Skipping."
        )
    except Exception as e:
        print(
            f"  ❌ Error al copiar '{item_name_in_package}': {e}. / "
            f"Error copying '{item_name_in_package}': {e}."
        )


def main():
    destination_base_path = Path(os.getcwd())
    project_dir_name = os.path.basename(destination_base_path) # Get the name of the current directory

    if list(destination_base_path.glob('*')):
        print(
            "⚠️ Advertencia: El directorio actual no está vacío. Los archivos del proyecto Lila se copiarán aquí. / "
            "Warning: The current directory is not empty. Lila project files will be copied here."
        )
        input("Presiona Enter para continuar o Ctrl+C para cancelar. / Press Enter to continue or Ctrl+C to cancel...")

    try:
        print(
            f"\n🛠️  Construyendo el esqueleto del proyecto en el directorio actual usando Lila Framework... / "
            f"Scaffolding project in the current directory using Lila Framework..."
        )

        for item_pkg_name, item_dest_name in SCAFFOLD_ITEMS.items():
            copy_item("lila", item_pkg_name, destination_base_path, item_dest_name)

        scaffolded_env_example_name = SCAFFOLD_ITEMS.get("env_example", "env_example")
        env_example_in_project_path = (
            destination_base_path / scaffolded_env_example_name
        )
        env_path = destination_base_path / ".env"

        if env_example_in_project_path.exists() and not env_path.exists():
            shutil.copy2(env_example_in_project_path, env_path)
            print(
                f"\n✨ '{scaffolded_env_example_name}' copiado a '.env'. ¡No olvides configurarlo! / "
                f"'{scaffolded_env_example_name}' copied to '.env'. Don't forget to configure it!"
            )
        elif env_example_in_project_path.exists() and env_path.exists():
            print(
                f"\nℹ️ Ya existe un archivo '.env'. Se copió igualmente '{scaffolded_env_example_name}'. / "
                f"An '.env' file already exists. '{scaffolded_env_example_name}' was also copied."
            )
        elif not env_example_in_project_path.exists():
            print(
                f"\n⚠️ Advertencia: '{scaffolded_env_example_name}' no se encontró en el proyecto scaffolded, no se pudo crear '.env' automáticamente. / "
                f"Warning: '{scaffolded_env_example_name}' not found in the scaffolded project, '.env' could not be created automatically."
            )

        readme_content = f"""
# {project_dir_name.replace('_', ' ').title()}

# Learning Lila
https://seip25.github.io/Lila 

# Documentación de Lila
https://seip25.github.io/Lila

¡Feliz desarrollo! 🚀
"""
        with open(destination_base_path / "README.md", "w",encoding="utf-8") as f:
            f.write(readme_content)
        print(
            f"📄 README.md básico creado en el directorio actual. / "
            f"Basic README.md created in the current directory."
        )

        print(
            "\n🎉 ¡Proyecto Lila inicializado exitosamente en el directorio actual! / "
            "Lila project initialized successfully in the current directory!"
        )
        print(
            "➡️  Ahora, revisa los archivos generados en este directorio. / "
            "Now, review the generated files in this directory."
        )
        print(
            "   Sigue las instrucciones en el README.md generado. / "
            "Follow the instructions in the generated README.md."
        )

    except Exception as e:
        print(
            f"\n❌ Ocurrió un error durante la creación del proyecto: {e}. / "
            f"An error occurred during project creation: {e}."
        )
        if destination_base_path.exists() and not any(destination_base_path.iterdir()): # Only try to cleanup if the directory was just created and is empty
            try:
                destination_base_path.rmdir()
                print(
                    f"🧹 Directorio del proyecto incompleto eliminado: {destination_base_path}. / "
                    f"Incomplete project directory deleted: {destination_base_path}."
                )
            except Exception as cleanup_e:
                print(
                    f"⚠️ No se pudo eliminar automáticamente el directorio '{destination_base_path}': {cleanup_e}. Por favor, elimínalo manualmente. / "
                    f"Could not automatically delete directory '{destination_base_path}': {cleanup_e}. Please delete it manually."
                )


if __name__ == "__main__":
    print(
        "Este script está pensado para ejecutarse como 'lila-init' después de instalar 'lila-framework'. / "
        "This script is intended to be run as 'lila-init' after installing 'lila-framework'."
    )



from sqlalchemy import create_engine, MetaData, text, inspect
from sqlalchemy.exc import SQLAlchemyError
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import sessionmaker, Session
from typing import Optional
from lila.core.logger import Logger

Base = declarative_base()


class Database:
    def __init__(self, config: dict) -> None:
        self.type = config.get("type", "sqlite")
        self.config = config
        self.connection = None
        self.metadata = MetaData()
        self.tables = []
        self.auto_commit = self.config.get("auto_commit", False)
        self.engine = None
        self.SessionLocal = None

    def connect(self) -> bool:
        if self.type in ["mysql", "postgresql", "psgr"]:
            user = self.config.get("user", "root")
            password = self.config.get("password", "")
            host = self.config.get("host", "127.0.0.1")
            port = self.config.get("port", 3306)
            database = self.config.get("database", "db")
            db_type = "postgresql" if self.type in ["postgresql", "psgr"] else self.type
            connector = "mysqlconnector" if self.type == "mysql" else "psycopg2"
            isolation_level = self.config.get("isolation_level", None)

            temp_engine = create_engine(
                f"{db_type}+{connector}://{user}:{password}@{host}:{port}/postgres"
                if db_type == "postgresql"
                else f"{db_type}+{connector}://{user}:{password}@{host}:{port}/mysql"
            )

            with temp_engine.connect() as connection:
                if db_type == "postgresql":
                    result = connection.execute(
                        text(f"SELECT 1 FROM pg_database WHERE datname = '{database}'")
                    )
                else:  # MySQL
                    result = connection.execute(
                        text(f"SHOW DATABASES LIKE '{database}'")
                    )

                if not result.fetchone():

                    connection.execute(text(f"CREATE DATABASE {database}"))
                    print(f"Database '{database}' created.")

            try:
                self.engine = create_engine(
                    f"{db_type}+{connector}://{user}:{password}@{host}:{port}/{database}",
                    isolation_level=isolation_level,
                    execution_options={"autocommit": self.auto_commit},
                )
                self.SessionLocal = sessionmaker(
                    autocommit=False, autoflush=False, bind=self.engine
                )
            except SQLAlchemyError as e:
                print(f"Database connection error: {e}")
                return False

        else:
            database = self.config.get("database", "db")
            try:
                self.engine = create_engine(f"sqlite:///{database}.sqlite")
                self.SessionLocal = sessionmaker(
                    autocommit=False, autoflush=False, bind=self.engine
                )
            except SQLAlchemyError as e:
                print(f"Create database, error: {e}")
                Logger.error(f"Create database, error: {e}")
                return False
        return True

    def get_session(self) -> Session:
        if not self.SessionLocal:
            raise Exception("Database not connected. Call `connect()` first.")
        return self.SessionLocal()

    def prepare_migrate(self, tables: list) -> None:
        self.tables.extend(tables)

    def migrate(self, use_base: bool = False):
        try:
            if use_base:
                Base.metadata.create_all(self.engine)
            else:
                self.metadata.create_all(self.engine)
            print("success migrations")
        except SQLAlchemyError as e:
            Logger.error(f"Error database.py: {e}")
            print(e)

    def query(
        self,
        query: str,
        params: Optional[dict] = None,
        return_rows: bool = False,
        return_row: bool = False,
    )-> dict | bool | list | None:
        result = False
        try:
            with self.engine.connect() as connection:
                result = connection.execute(text(query), params or ())
                if (
                    query.strip()
                    .upper()
                    .startswith(("CREATE", "INSERT", "UPDATE", "DELETE"))
                ):
                    connection.commit()
                if return_rows:
                    if result:
                        rows = result.fetchall() if result else []
                        items = [dict(getattr(item, "_mapping", {})) for item in rows]
                        return items
                    return []
                if return_row:
                    if result:
                        row = result.fetchone() if result else None
                        if row:
                            item = dict(getattr(row, "_mapping", {}))
                            return item
                    return None
                return result
        except SQLAlchemyError as e:
            print(f"Query error: {e}")
            Logger.error(f"Query error: {e}")
        return result

    def commit(self) -> None:
        if self.connection and not self.auto_commit:
            try:
                self.connection.commit()
            except SQLAlchemyError as e:
                Logger.error(f"Commit error: {e}")
                print(f"Commit error: {e}")

    def close(self) -> None:
        if self.connection:
            try:
                self.connection.close()
                self.connection = None
                print("Connection closed.")
            except SQLAlchemyError as e:
                print(f"Close connection error: {e}")



from starlette.templating import Jinja2Templates
from app.config import VERSION_PROJECT,TITLE_PROJECT
from app.helpers.helpers import theme,lang,translate as t
from lila.core.request import Request
from lila.core.responses import HTMLResponse ,JSONResponse
from app.helpers.helpers import lang
from app.config import PATH_TEMPLATES_HTML,PATH_TEMPLATES_MARKDOWN
from lila.core.logger import Logger
import markdown
import os
import traceback
import sys


templates = Jinja2Templates(directory=PATH_TEMPLATES_HTML)
 
def render(request:Request, template: str,context :dict ={},theme_ :bool= True,translate:bool = True,files_translate:list=[],lang_default:str=None,templates=templates):
    try:
        template = f"{template}.html"
        default_context = {
            'title'   :TITLE_PROJECT,
            'version' : VERSION_PROJECT
        }
        if theme_:
            default_context['theme']=theme(request=request)

        if translate:
            if lang_default:
                default_context['lang']=lang_default
            else:
                default_context['lang']=lang(request=request)
            default_context['translate']=t(file_name='translations',request=request,lang_default=lang_default)
            
            if len(files_translate) > 0:
                for file_name in files_translate:
                    add_translations=t(file_name=file_name,request=request,lang_default=lang_default)
                    default_context['translate'].update(add_translations)
        context.update(default_context)
        return templates.TemplateResponse(request=request,name=template,context=context)
    except Exception as e:
        
        exc_type, exc_value, exc_tb = sys.exc_info()
        
        error_info = {
            'error_type': exc_type.__name__,
            'error_message': str(e),
            'template': template,
            'traceback': traceback.format_exc()
        }
         
        if exc_tb:
            frame = exc_tb.tb_frame
            error_info['file'] = frame.f_code.co_filename
            error_info['line'] = exc_tb.tb_lineno
            error_info['function'] = frame.f_code.co_name

           
            
            Logger.error(f"Render error in {error_info['file']}:{error_info['line']} "
                        f"({error_info['function']}): {error_info['error_message']}")
        else:
            Logger.error(f"Render error: {error_info['error_message']}") 

        error_details = f"{error_info['error_type']}: {error_info['error_message']}"
        message=  "Error rendering template"
        print(f" {message} - {error_details}")
        return JSONResponse({
            "success": False, 
            "message":message,
            "error_details":error_details
        }, status_code=500)
    
def renderMarkdown(request,file : str , base_path:str =PATH_TEMPLATES_MARKDOWN,css_files : list = [],js_files:list=[],picocss : bool =True):
    file_path=os.path.join(base_path,f"{file}.md")
    if not os.path.exists(file_path):
        return HTMLResponse('<h5>404</h5><br/><p>Not found</p>')
    
    with open(file=file_path,mode="r",encoding="utf-8") as file:
        mark=file.read()
        html_markdown= markdown.markdown(mark)   
    
    lang_=lang(request)
    
    title=TITLE_PROJECT
    head='<!DOCTYPE html>\n'
    head+=f"<html lang='{lang_}'>\n"
    head+='<head>\n'
    head+='<meta charset="UTF-8">\n'
    head+='<meta name="viewport" content="width=device-width, initial-scale=1.0">\n'
    head+='<meta name="color-scheme" content="light dark">\n'
    head+='<meta http-equiv="X-UA-Compatible" content="ie=edge">\n'
    head+=f"<title>{title}</title>\n"
    if  picocss:
       head+= '<link  rel="stylesheet"  href="/public/css/pico.css">'
    if css_files :
        for css in css_files:
            head+=f"<link rel='stylesheet' type='text/css' href='{css}' />\n"
    
    if js_files :
        for js in js_files:
            head+=f"<script src='{js}' ></script>\n"
    
    head+='</head>\n<body>\n'

 
    html=f"{head}<main class='container'>{html_markdown}</main></body>\n</html>"    
    
    return HTMLResponse(content=html)



from dotenv import load_dotenv
from os import getenv, path
import os

env_path = path.join(os.getcwd(), ".env")
load_dotenv(dotenv_path=env_path, encoding="utf-8")
SECRET_KEY = getenv("SECRET_KEY", "")
PORT = int(getenv("PORT", 8001))
HOST = getenv("HOST", "127.0.0.1")
DEBUG = getenv("DEBUG", "True").lower() in ("true", "1", "yes")



PATH_LOG_BASE_DIR = "app/logs"
PATH_SECURITY = "app/security/"
PATH_TEMPLATE_NOT_FOUND = "templates/html/404.html"
PATH_TEMPLATES_HTML ="templates/html/"
PATH_TEMPLATES_MARKDOWN = "templates/markdown/"
PATH_LOCALES = "app/locales"
PATH_UPLOADS='/static/img/uploads'


#Project
TITLE_PROJECT = "Lila project"
VERSION_PROJECT = 1
DESCRIPTION_PROJECT = ""
THEME_DEFAULT = "blue"
LANG_DEFAULT = "en"


#Security
SENSITIVE_PATHS = [
    "/admin",
    "/config",
    "/env",
    "/.env",
    "/.git",
    "/wp-admin",
    "/wp-login",
    "/database",
    "/backup",
]



from sqlalchemy import Table,Column,Integer,String,TIMESTAMP
from app.connections import connection
from lila.core.database import Base #Import Base  for migrations in models
from app.models.user import User #Import models for migrations in models
import typer
import asyncio


app = typer.Typer()

#English:Example of creating migrations for 'users'
#Español: Ejemplo de creación de migraciones para 'users'
# table_users = Table(
#     'users', connection.metadata,
#     Column('id', Integer, primary_key=True,autoincrement=True),
#     Column('name', String(length=50), nullable=False),
#     Column('email', String(length=50), unique=True),
#     Column('password', String(length=150), nullable=False),
#     Column('token', String(length=150), nullable=False),
#     Column('active', Integer,default=1, nullable=False),
#     Column('created_at', TIMESTAMP), 
# )

async def migrate_async(connection,refresh:bool=False)->bool:
    
    try:
        if refresh:
            connection.metadata.drop_all(connection.engine)
        # connection.prepare_migrate([table_users])#for tables
        # connection.migrate() 
        connection.migrate(use_base=True)#for models , always import models in the file
        print("Migrations completed")
        
        return True
    except RuntimeError as e:
        print(e)
    
    
@app.command()
def migrate(refresh: bool = False):
    """Run database migrations"""
    success = asyncio.run(migrate_async(connection, refresh))
    if not success:
        raise typer.Exit(code=1)

if __name__ == "__main__":
    app()


from lila.core.app import App
from app.routes.routes import routes
from app.routes.api import routes as api_routes
from lila.core.middleware import Middleware
from app.config import PORT, HOST, DEBUG
from app.middlewares.defaults import (
    LoggingMiddleware,
    IPBlockingMiddleware,
    URLBlockingMiddleware,
    MaliciousExtensionMiddleware,
    SensitivePathMiddleware,
    ErrorHandlerMiddleware,
)
import itertools
import uvicorn
import asyncio

# English: Combining application and API routes into a single list.
# Español: Combinando las rutas de la aplicación y la API en una única lista.
all_routes = list(itertools.chain(routes, api_routes))

# English: Here we activate the admin panel with default settings.
# Español: Aquí activamos el panel de administrador con configuraciones predeterminadas.
# from app.routes.admin import Admin
# from app.models.user import User
# admin_routes=Admin(models=[User])
# all_routes = list(itertools.chain(routes, api_routes,admin_routes))
# English: Marker for the admin routes in main.py.
# Español: Marcador para las rutas de administrador en main.py.
# admin_marker

cors = None

# English: CORS usage example
# Español : Ejemplo de utilización de CORS
# cors={
#     "origin": ["*"],
#     "allow_credentials" : True,
#     "allow_methods":["*"],
#     "allow_headers": ["*"]
# }
# app = App(debug=True, routes=all_routes,cors=cors)

# English:necessary for cli command modify react cors for development
# Español:necesario para el comando cli modificar cors de react para desarrollo
# react_marker

middlewares = [
    Middleware(LoggingMiddleware),
    Middleware(IPBlockingMiddleware),
    Middleware(URLBlockingMiddleware),
    Middleware(MaliciousExtensionMiddleware),
    Middleware(SensitivePathMiddleware),
    Middleware(ErrorHandlerMiddleware),
]

# English: Initializing the application with debugging enabled and the combined routes.
# Español: Inicializando la aplicación con la depuración activada y las rutas combinadas.
app = App(debug=DEBUG, routes=all_routes, cors=cors, middleware=middlewares)

# English: To ensure SEO (bots, AI), caching, and HTML hydration, uncomment these lines.
# Adding {% include "react/cache/index.html" %} to <div id="root">
# Español :Para tener seo (bots,ia) ,cache y que react hidrate el html ,descomenta estas lineas.
# Agregadando en <div id="root"> ,  {% include "react/cache/index.html"  %}

# import subprocess
# import sys

# @app.on_event("startup")
# async def startup_event():
#     print("♻️ Prerender for react...")
#     url_with_port =f" http://{HOST}:{PORT}"
#     subprocess.Popen([
#     sys.executable,
#     "-m",
#     "cli.prerender",
#     "--url",
#     url_with_port.strip()
# ])


# English: Asynchronous main function to run the application server.
# Español: Función principal asíncrona para ejecutar el servidor de la aplicación.
async def main():
    # English: Starting the Uvicorn server with the application instance.
    # Español: Iniciando el servidor Uvicorn con la instancia de la aplicación.
    uvicorn.run("main:app", host=HOST, port=PORT, reload=True)


# English: Entry point for the script, running the main asynchronous function.
# Español: Punto de entrada del script, ejecutando la función principal asíncrona.
if __name__ == "__main__":
    try:
        asyncio.run(main())
    except KeyboardInterrupt:
        # English: Gracefully shutting down the application on keyboard interrupt.
        # Español: Apagando la aplicación de manera ordenada al interrumpir con el teclado.
        print("Shutting down the application...")
        pass



from starlette.routing import Route, Mount
from starlette.staticfiles import StaticFiles
from lila.core.responses import HTMLResponse, JSONResponse
from lila.core.request import Request
from app.config import TITLE_PROJECT, VERSION_PROJECT, DESCRIPTION_PROJECT
from typing import Any, Type, Optional, List
from pydantic import BaseModel
from argon2 import PasswordHasher
from app.helpers.helpers import generate_token_value, get_user_by_token
from lila.core.logger import Logger
import datetime
import re
from functools import wraps

ph = PasswordHasher()


class Router:
    def __init__(self) -> None:
        self.routes = []
        self.docs = []

    def route(self, path: str, methods: list[str] = ["GET"], model=None) -> None:
        try:

            def decorator(func):
                self.routes.append(Route(path=path, endpoint=func, methods=methods))
                if model is not None:
                    self.docs.append({"path": path, "model": model})

                return func

            return decorator
        except RuntimeError as e:
            Logger.error(f"Error : {str(e)}")
            print(f"Error : {e}")

    def mount(
        self, path: str = "/public", directory: str = "static", name: str = "static"
    ) -> None:

        try:
            self.routes.append(Mount(path, StaticFiles(directory=directory), name=name))
        except RuntimeError as e:
            Logger.error(f"Error : {str(e)}")
            print(f"Error :{e}")

    def get_routes(self) -> list:
        return self.routes

    def swagger_ui(self, path: str = "/docs", methods: list[str] = ["GET"]) -> None:
        def swagger_docs(request: Request):
            html = """
                <html>
                <head>
                    <title>API Documentation</title>
                    <link rel="stylesheet" type="text/css" href="/public/swagger/swagger-ui.css" />
                    <script src="/public/swagger/swagger-ui-bundle.js"></script>
                    <script src="/public/swagger/swagger-ui-standalone-preset.js"></script>
                </head>
                <body>
                    <div id="swagger-ui"></div>
                  
                    <script>
                        const ui = SwaggerUIBundle({
                            url: '/openapi.json',  
                            dom_id: '#swagger-ui',
                            deepLinking: true,
                            presets: [
                                SwaggerUIBundle.presets.apis,
                                SwaggerUIStandalonePreset
                            ],
                            layout: "StandaloneLayout"
                        });
                    </script>
                </body>
            </html>
            """
            return HTMLResponse(html)

        self.routes.append(Route(path=path, endpoint=swagger_docs, methods=methods))

    def openapi_json(
        self, path: str = "/openapi.json", methods: list[str] = ["GET"]
    ) -> None:
        def openapi_schema(request: Request):
            openapi_schema = {
                "openapi": "3.0.0",
                "info": {
                    "title": TITLE_PROJECT,
                    "version": VERSION_PROJECT,
                    "description": DESCRIPTION_PROJECT,
                },
                "paths": {},
            }

            for route in self.routes:
                if isinstance(route, Route):
                    methods = route.methods or ["GET"]
                    path = route.path
                    if path not in ["/docs", "/openapi.json"]:
                        for method in methods:
                            if path not in openapi_schema["paths"]:
                                openapi_schema["paths"][path] = {}
                            if method.lower() != "head":
                                openapi_schema["paths"][path][method.lower()] = {
                                    "summary": f"{route.name} ",
                                    **(
                                        {
                                            "requestBody": {
                                                "content": {
                                                    "application/json": {
                                                        "schema": {
                                                            "type": "object",
                                                            "properties": self.get_params(
                                                                route
                                                            ),
                                                        }
                                                    }
                                                }
                                            }
                                        }
                                        if method.lower() not in ["get", "delete"]
                                        else {}
                                    ),
                                    "parameters": [
                                        {
                                            "name": param,
                                            "in": "path",
                                            "required": True,
                                            "schema": {"type": "integer"},
                                        }
                                        for param in self.get_path_params(path)
                                    ],
                                    "responses": {
                                        "200": {
                                            "description": route.endpoint.__doc__
                                            or f"{route.name} function"
                                        }
                                    },
                                }

            return JSONResponse(openapi_schema)

        self.routes.append(Route(path=path, endpoint=openapi_schema, methods=methods))

    def get_path_params(self, path: str):
        matches = re.findall(r"{(\w+)(?::\w+)?}", path)
        return matches

    def get_params(self, route):
        model = None
        parameters = {}
        for doc in self.docs:
            if doc["path"] == route.path:
                model = doc["model"]
                break

        if model:
            fields = model.__fields__
            for field, field_info in fields.items():
                field_type = self.pydantic_type_to_openapi(field_info=field_info)
                parameters[field] = {
                    "name": field,
                    "required": field_info.required,
                    "name": field,
                    "type": field_type,
                }

        return parameters

    def pydantic_type_to_openapi(self, field_info: Any) -> str:
        if field_info.type_ is str:
            return "string"
        elif field_info.type_ is int:
            return "integer"
        elif field_info.type_ is bool:
            return "boolean"
        elif field_info.type_ is float:
            return "number"
        return "string"

    def rest_crud_generate(
        self,
        router,
        connection,
        model_sql,
        model_pydantic: Type[BaseModel],
        select: Optional[List[str]] = None,
        columns: Optional[List[str]] = None,
        active: bool = False,
        delete_logic: bool = False,
        middlewares: dict = None,
        jsonresponse_prefix: str = "",
        user_id_session: bool | str = False,
        base_path: str = None,
    ) -> None:
        name = base_path or f"/api/{model_sql.__tablename__}"
        crud_routes=[]
        
        def middleware(func):
            @wraps(func)
            async def middleware_wr(*args, **kwargs):
                if callable(func):
                    result = func(*args, **kwargs)
                    if hasattr(result, "__await__"):
                        return await result
                    return result

            return middleware_wr

        async def execute_middleware(self, type: str):
            if middlewares is not None and type in middlewares:
                for middleware_get in middlewares[type]:
                    middleware_func = middleware(middleware_get)
                    response = await middleware_func(self)
                    if isinstance(response, JSONResponse):
                        return response

        def get_user_id_session(self):
            id_token = get_user_by_token(self)
            if isinstance(id_token, JSONResponse):
                return id_token
            return id_token

        async def get(self):
            try:
                response = await execute_middleware(self, type="get")
                if isinstance(response, JSONResponse):
                    return response

                columns = " , ".join(select) if select else "*"
                filters = f"WHERE active = 1" if active else ""
                params = {"user_id": 0}
                if user_id_session:

                    user_id = get_user_id_session(self)
                    if isinstance(user_id, JSONResponse):
                        return user_id
                    params[user_id_session] = user_id
                    filters += f" AND {user_id_session}= :{user_id_session}"
                else:
                    params = None

                query = f"SELECT {columns} FROM {model_sql.__tablename__} {filters}"
                items = connection.query(query=query, params=params, return_rows=True)
                return (
                    JSONResponse(items)
                    if jsonresponse_prefix == ""
                    else JSONResponse({jsonresponse_prefix: items})
                )
            except Exception as e:
                Logger.error(f"Error rest crud , GET: {str(e)}")

        async def post(self):
            response = await execute_middleware(self, type="post")
            if isinstance(response, JSONResponse):
                return response
            try:
                body = await self.json()
                model = model_pydantic(**body)
            except Exception as e:
                print(str(e))
                Logger.warning(f"Error rest_crud_generate - POST: {str(e)}")
                return JSONResponse({"success": False, "msg": str(e)}, status_code=400)
            columns_ = (
                " , ".join(columns)
                if columns
                else " , ".join((row) for row in list(model_pydantic.__fields__.keys()))
            )
            values = (
                " , ".join(f":{row}" for row in columns)
                if columns
                else " , ".join(
                    f":{row}" for row in list(model_pydantic.__fields__.keys())
                )
            )
            params = {
                field: getattr(model, field)
                for field in model_pydantic.__fields__.keys()
            }
            if "token" in params:
                params["token"] = generate_token_value()
            if "hash" in params:
                params["hash"] = generate_token_value()

            if "password" in params:
                params["password"] = ph.hash(params["password"])

            if "created_at" in body:
                params["created_at"] = datetime.datetime.now().strftime(
                    "%Y-%m-%d %H:%M:%S"
                )
                columns_ += ",created_at"
                values += ",:created_at"

            if "created_date" in body:
                params["created_date"] = datetime.datetime.now().strftime(
                    "%Y-%m-%d %H:%M:%S"
                )
                columns_ += ",created_date"
                values += ",:created_date"

            if user_id_session:
                user_id = get_user_id_session(self)
                if isinstance(user_id, JSONResponse):
                    return user_id
                if user_id is not None:
                    params[user_id_session] = user_id
                    columns_ += f", {user_id_session}"
                    values += f", :{user_id_session}"

            if active:
                params["active"] = 1
                columns_ += ", active"
                values += ", :active"

            query = (
                f" INSERT INTO {model_sql.__tablename__} ({columns_}) VALUES ({values})"
            )
            try:
                result = connection.query(query=query, params=params)
                id = result.lastrowid if result else 0
                result = True if result else False
                status_code = 201 if result else 200
                return JSONResponse(
                    {"success": result, "id": id}, status_code=status_code
                )
            except Exception as e:
                print(e)
                Logger.error(f"Error rest_crud_generate , POST: {str(e)}")
                return JSONResponse({"success": False}, status_code=500)

        def search_id(self) -> bool | dict:
            columns = " , ".join(select) if select else "*"
            filters = f"active = 1" if active else ""
            filters += " AND  id =:id" if filters else "  id = :id"
            if "user_id" in model_pydantic.__fields__.keys():
                filters += " AND user_id:user_id"
            elif "id_user" in model_pydantic.__fields__.keys():
                filters += " AND id_user:id_user"

            id = int(self.path_params["id"])
            params = {"user_id": 0, "id": id}

            if user_id_session:
                user_id = get_user_id_session(self)
                if isinstance(user_id, JSONResponse):
                    return user_id
                if user_id is not None:
                    params[user_id_session] = user_id
                    filters += f" AND {user_id_session} = :{user_id_session}"

            query = f"SELECT {columns} FROM {model_sql.__tablename__} WHERE {filters} "
            results = connection.query(query=query, params=params, return_row=True)
            return results

        async def get_id(self) -> dict:
            response = await execute_middleware(self, type="get_id")
            if isinstance(response, JSONResponse):
                return response
            item = search_id(self)
            if item is None:
                return JSONResponse({}, status_code=404)
            return (
                JSONResponse(item)
                if jsonresponse_prefix == ""
                else JSONResponse({jsonresponse_prefix: item})
            )

        async def put(self):
            response = await execute_middleware(self, type="put")
            if isinstance(response, JSONResponse):
                return response
            result = search_id(self)
            if result is None:
                return JSONResponse({"success": False}, status_code=404)
            result_update = False
            try:
                body = await self.json()
                model = model_pydantic(**body)
            except Exception as e:
                print(str(e))
                return JSONResponse({"success": False, "msg": str(e)}, status_code=400)

            values = (
                "  ".join(f"{row}= :{row}" for row in columns)
                if columns
                else " , ".join(
                    f"{row}=:{row}" for row in list(model_pydantic.__fields__.keys())
                )
            )
            params = {
                field: getattr(model, field)
                for field in model_pydantic.__fields__.keys()
            }

            id = int(self.path_params["id"])
            params["id"] = id
            if "password" in params:
                params["password"] = ph.hash(params["password"])

            if "token" in params:
                params["token"] = generate_token_value()
            if "hash" in params:
                params["hash"] = generate_token_value()

            filters = ""

            if user_id_session:
                user_id = get_user_id_session(self)
                if isinstance(user_id, JSONResponse):
                    return user_id
                if user_id is not None:
                    params[user_id_session] = user_id
                    filters += f" AND {user_id_session} = :{user_id_session}"

            query = f" UPDATE {model_sql.__tablename__} SET {values} WHERE id= :id {filters}"
            id = self.path_params["id"]
            params["id"] = int(id)
            result = connection.query(query=query, params=params)
            result_update = True if result else False
            return JSONResponse({"success": result_update})

        async def delete(self):
            response = await execute_middleware(self, type="delete")
            if isinstance(response, JSONResponse):
                return response
            result = search_id(self)
            if result is None:
                JSONResponse({"success": False}, status_code=404)

            id = int(self.path_params["id"])
            params = {"id": id, "user_id": 0}
            filters = ""
            if user_id_session:
                user_id = get_user_id_session(self)
                if isinstance(user_id, JSONResponse):
                    return user_id
                if user_id is not None:
                    params[user_id_session] = user_id
                    filters += f" AND {user_id_session} = :{user_id_session}"

            if delete_logic:
                query = f" UPDATE {model_sql.__tablename__} SET active=0 WHERE id= :id {filters}"
            else:
                query = f"DELETE FROM {model_sql.__tablename__} WHERE id=:id {filters}"

            result = connection.query(query=query, params=params)
            result_delete = True if result else False
            return JSONResponse({"success": result_delete})

        crud_routes += [
            Route(path=name, name=name, methods=["GET"], endpoint=get),
            Route(path=name, name=name, methods=["POST"], endpoint=post),
            Route(
                path=f"{name}/{{id}}",
                name=f"{name}_get_id",
                methods=["GET"],
                endpoint=get_id,
            ),
            Route(
                path=f"{name}/{{id}}",
                name=f"{name}_put",
                methods=["PUT"],
                endpoint=put,
            ),
            Route(
                path=f"{name}/{{id}}",
                name=f"{name}_delete",
                methods=["DELETE"],
                endpoint=delete,
            ),
        ]
        docs_info = [
            {"path": name, "model": model_pydantic},
            {"path": f"{name}/{{id}}", "model": model_pydantic},
        ]
        router.routes.extend(crud_routes)
        router.docs.extend(docs_info)


import json
from app.config import SECRET_KEY
from itsdangerous import BadSignature, URLSafeTimedSerializer,SignatureExpired
from lila.core.request import Request
from typing import Any, Union, Optional, Dict, List
from lila.core.logger import Logger

serializer = URLSafeTimedSerializer(SECRET_KEY)


class Session:
    @staticmethod
    def setSession(
        new_val: str | dict | list,
        response,
        name_cookie: str = "session",
        secure: bool = True,
        samesite: str = "strict",
        max_age: int = 3600,
        domain: Optional[str] = None,
        http_only: bool = True,
        path: str = "/",
    ) -> bool:
        try:
            if isinstance(new_val, (dict, list)):
                new_val = json.dumps(new_val)
            else:
                new_val = str(new_val)

            signed_session = serializer.dumps(new_val)

            response.set_cookie(
                key=name_cookie,
                value=signed_session,
                max_age=max_age,
                expires=max_age,
                secure=secure,
                httponly=http_only,
                samesite=samesite,
                domain=domain,
                path=path,
            )
            return True
        except (TypeError, ValueError, Exception) as e:
            Logger.error(f"Error setting session: {str(e)}")
            print(f"Error session {str(e)}")
            return False

    @staticmethod
    def getSession(key: str, request: Request) -> Optional[str]:
        return request.cookies.get(key)

    @staticmethod
    def unsign(
        key: str, request: Request, max_age: Optional[int] = None
    ) -> Union[Dict, List, str, None]:
        session_data = request.cookies.get(key)
        if not session_data:
            return None

        try:
            unsigned_data = serializer.loads(session_data, max_age=max_age)
            try:
                return json.loads(unsigned_data)
            except json.JSONDecodeError:
                return unsigned_data

        except BadSignature:
            msg = f"Invalid session signature for cookie: {key}"
            print(msg)
            Logger.warning(msg)
            return None
        except Exception as e:
            msg = f"Error unsigning session: {str(e)}"
            print(msg)
            Logger.error(msg)
            return None

    @staticmethod
    def deleteSession(response, name_cookie: str) -> bool:
        try:
            response.delete_cookie(name_cookie)
            return True
        except Exception as e:
            msg = f"Error deleting session: {str(e)}"
            print(e)
            Logger.error(msg)
            return False
        
    @staticmethod
    def getSessionValue(
         request: Request,key: str='auth', max_age: Optional[int] = 3600
    ) -> Union[Dict, List, str, None]:
        session_data = request.cookies.get(key)
        if not session_data:
            return None

        try:
            unsigned_data = serializer.loads(session_data, max_age=max_age)
            try:
                return json.loads(unsigned_data)
            except json.JSONDecodeError:
                return unsigned_data

        except BadSignature:
            msg = f"Invalid session signature for cookie: {key}"
            print(msg)
            Logger.warning(msg)
            return None
        except SignatureExpired:
            msg = f"Session expired for cookie: {key}"
            print(msg)
            Logger.warning(msg)
            return None
        except Exception as e:
            msg = f"Error unsigning session: {str(e)}"
            print(msg)
            Logger.error(msg)
            return None



from starlette.responses import  JSONResponse as StarletteJSONResponse,HTMLResponse,RedirectResponse,PlainTextResponse,RedirectResponse,StreamingResponse
from decimal import Decimal
from datetime import date, datetime

 
HTMLResponse = HTMLResponse
RedirectResponse = RedirectResponse
PlainTextResponse = PlainTextResponse
RedirectResponse=RedirectResponse
StreamingResponse=StreamingResponse

def convert_to_serializable(obj):
    if isinstance(obj, (Decimal)):
        return float(obj) 
    elif isinstance(obj, (date, datetime)):
        return obj.isoformat()  
    elif isinstance(obj, dict):
        return {key: convert_to_serializable(value) for key, value in obj.items()}
    elif isinstance(obj, (list, tuple)):
        return [convert_to_serializable(item) for item in obj]
    elif hasattr(obj, "__dict__"):  
        return convert_to_serializable(obj.__dict__)
    else:
        return obj 

class JSONResponse(StarletteJSONResponse):
    def __init__(self, data, status_code=200,serialize=True):
        serialized_data = convert_to_serializable(data) if serialize else data
        super().__init__(serialized_data, status_code=status_code)



from app.helpers.helpers import LANG_DEFAULT
from lila.core.session import Session
from lila.core.request import Request

class Translate:
   
    def setLang(self,new_lang : str,response )-> None:
        lang ={'lang':new_lang}
        Session.setSession(new_lang=lang,response=response)
        
    def getLang(self,request : Request)->str :
        session_lang=Session.getSessionValue(key='lang',request=request)
        lang = session_lang if session_lang else LANG_DEFAULT
        return lang